/* Database creation, insertion, and population routines
 *   By Charles Shiflett
 *
 * set ts=4 									( 4 character tabs )
 */

#include "post.h"
#include "ieee.h"

 /* init_databases()
  *  Creates the default tables used for the IEEE database 
  */

void init_database() {

    /* Note: order is important */

	exec("create table inventory (entry_num SERIAL primary key, type text, sub_type text,
			 size TEXT, color TEXT, def_price NUMERIC(24,2), stock INTEGER, 
			 description TEXT, ds TIMESTAMP)");

	exec("create table person (person_num SERIAL primary key, ieee_num INTEGER, 
			 real_name TEXT, user_name TEXT, password TEXT, phone INTEGER,
			 email TEXT, unique (ieee_num), unique(user_name),
			 street TEXT, zip INTEGER, city TEXT, url TEXT, comments TEXT[], 
	         description TEXT, ds TIMESTAMP)");

	exec("create table order_form (order_num SERIAL primary key, 
			 paid TEXT, delivered TEXT, who INTEGER references person, account TEXT,
	         description TEXT, ds TIMESTAMP)");

	exec("create table receipt (receipt_num SERIAL primary key, order_num INTEGER references order_form, 
	         item_num INTEGER references inventory, price NUMERIC(24,2), taxable INTEGER, 
	         description TEXT, ds TIMESTAMP)");

	exec("create table expense (expense_num SERIAL primary key, amount NUMERIC(24,2), 
	         account TEXT, who INTEGER references person, description TEXT, ds TIMESTAMP)");
}

/* Basic insert routines used in ieee database
 *
 *  form is func(struct, foreign keys, ... )
 */

void insert_inventory(inventory_type* inv) {
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, 
	    "INSERT INTO inventory (type, sub_type, size, color, def_price, stock, description, ds) values ('%s','%s','%s','%s','%d.%d','%d','%s',now())",
		inv->type, inv->sub_type, inv->size, inv->color,
		inv->price /100, inv->price % 100 , inv->stock, inv->description);
	exec(buf);
}

void insert_person(persocom* person) {
	char buf[BUF_SIZE], ieee_num[10], user_name[10];

	if (person->ieee_number)
		snprintf(ieee_num, 10, "'%d'", person->ieee_number);
	else 
		snprintf(ieee_num, 10, "NULL");

	if (person->user_name)
		snprintf(user_name, 10, "'%s'", person->user_name);
	else
		snprintf(user_name, 10, "NULL");



	snprintf(buf, BUF_SIZE, 
		"INSERT INTO person (ieee_num, real_name, user_name, password, phone, email, street, zip, city, url, description, ds) values (%s,'%s',%s,'%s','%d','%s','%s','%d','%s','%s','%s', now())",
		ieee_num, person->real_name, user_name, person->password, person->phone, person->email, person->street, person->zip, person->city, person->url, person->description);
	exec(buf);
}

void insert_order(order_type* order, int person) {
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, 
	    "INSERT INTO order_form (who, paid, delivered, account, description,ds) values ('%d','%s','%s','%s','%s',now())",
		person, order->paid, order->delivered, order->account, order->description);
	exec(buf);
}

void insert_receipt(receipt_type* receipt, int order, int inventory) {
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, 
		"INSERT INTO receipt (order_num, item_num, price, taxable, description,ds) values ('%d','%d','%d.%d','%d','%s',now())",
		order, inventory, receipt->price/100, receipt->price%100, receipt->tax, receipt->description);
	exec(buf);
}

void insert_expense(expense_type* expense, int person) {
	char buf[BUF_SIZE];
	snprintf(buf, BUF_SIZE, 
		"INSERT INTO expense ( who, account, amount, description, ds ) values ('%d','%s','%d.%d', '%s', now() )",
		person, expense->account, expense->amount/100, expense->amount%100, expense->description );
	exec(buf);
}

/* Creates data to populate database 
 *
 *    For populating the database with fictitous data, we are primarily
 *    interested in the relations, and not the content. For that reason,
 *	  only a few fields are filled in.
 */
void populate() {
	int random_number, i;

	/* constants for populate */
	int number_inventory=50,
		number_order=500,
		number_receipt=1500,
		number_person=250,
		number_expense=100;

	char tmp1[50], tmp2[50], tmp3[50], null_string[1]="";

	receipt_type receipt;
	expense_type expense;
	order_type	 order;
	inventory_type inventory;
	persocom	 person;


	/* populate order: inventory, person, order, receipt, expense */

	/* inventory loop */

	for (i=0; i<number_inventory; i++) {

		sprintf(tmp1, "%d", random() % (number_inventory/4));
		inventory.type=inventory.sub_type=inventory.size=tmp1;

		sprintf(tmp2, "%d", i);
		inventory.color=tmp2;

		inventory.price=random() % (10000-100) + 100;
		inventory.stock=random() % 20 + 2;
		inventory.description=null_string;

		insert_inventory(&inventory);
	}

	/* person loop */

	for (i=0; i<number_person; i++) {
		sprintf(tmp1, "%d", random() % (number_person/4));
		person.real_name=tmp1;

		person.city=person.password=person.email=person.street=
		                         person.url=person.description=null_string;
		(int)person.user_name=person.zip=person.ieee_number=person.phone=0;

		insert_person(&person);
	}

	/* order loop */

	for (i=0; i <number_order; i++) {
		random_number=random();
		sprintf(tmp1,"%d", (random_number & 0x1) ? 0 : 1);
		sprintf(tmp2,"%d", (random_number & 0x2) ? 0 : 1);
		sprintf(tmp3,"Petty Cash");

		order.paid=tmp1;
		order.delivered=tmp2;
		order.account=tmp3;
		order.description=null_string;

		insert_order(&order, (random_number>>2)%number_person +1);

	}

	/* receipt loop */

	for (i=0; i<number_receipt; i++) {
		/* set tax to either one or zero, and set price to .01-100$ */
		receipt.tax=(receipt.price=random() % 10000 + 1) & 1;
		receipt.description=null_string;
		insert_receipt(&receipt, random() % number_order+1, 
		                         random() % number_inventory+1);
	}

	/* expense loop */

	for (i=0; i < number_expense; i++) {

		sprintf(tmp1,"Petty Cash");
		expense.account=tmp1;

		expense.description=null_string;
		expense.amount=random() % 100000 + 1000;

		insert_expense(&expense, random() % number_person+1);
	}

}

int main() {

	/* Initilize our random number generator based on where in memory we are */
	char memory_location;	
	srand((unsigned int)&memory_location);

	init_database();
	populate();

	return(0);
}
